package Logaholic::IO::File;

use strict;
use warnings;

use IO::File;

#use IO::Zlib;
use IO::Compress::Gzip;
use IO::Uncompress::Gunzip;

# Implement alias for convenience
#*write = \&print;
*readline = \&getline;

# Implement missing implementation for IO::Zlib
# NOTE: this may not work for a system that falls back to external gzip program
#  but should be no problem for default Compress::Zlib implementation within
#  IO::Zlib
#*IO::Zlib::SEEK = sub{ return $_[0]->{'file'}->gzseek($_[1], $_[2]); };
#*IO::Zlib::TELL = sub{ return $_[0]->{'file'}->gztell($_[1]); };

# Normally we'd use the hash syntax, but that is not valid for 5.6
use constant MODE_READ  => 'r';
use constant MODE_WRITE => 'w';

sub new {
    my ( $class, @args ) = @_;
    if ( @args < 2 ) {
        die("$class requires a filename and mode to create an instance");
    }
    my $filename = $args[0];
    my $mode     = $args[1];
    if ( $mode !~ m/^[rw]/ ) {
        die("Invalid mode: $mode");
    }

    my $properties = {
        'filename' => undef,
        'fh'       => undef,
        'mode'     => undef,
    };

    my $self = bless $properties, $class;
    return $self->set_mode($mode)->set_filename($filename);
}

sub set_mode {
    my ( $self, $mode ) = @_;
    $self->{'mode'} = $mode;
    return $self;
}

sub set_filename {
    my ( $self, $filename ) = @_;
    $self->{'filename'} = $filename;
    return $self;
}

sub _is_gzip_file {
    my ($self) = @_;
    if ( !defined( $self->{'is_gzip'} ) ) {
        $self->{'is_gzip'} = ( $self->{'filename'} =~ /\.gz$/ ) ? 1 : 0;
    }
    return $self->{'is_gzip'};
}

sub open {
    my ($self) = @_;
    return 1 if ( defined( $self->{'fh'} ) );
    if ( $self->_is_gzip_file() ) {

    #       $self->{'fh'} = IO::Zlib->new($self->{'filename'}, $self->{'mode'});
        if ( $self->{'mode'} eq MODE_WRITE ) {
            $self->{'fh'} = IO::Compress::Gzip->new( $self->{'filename'} );
        }
        else {
            $self->{'fh'} = IO::Uncompress::Gunzip->new( $self->{'filename'} );
        }
    }
    else {
        $self->{'fh'} = IO::File->new( $self->{'filename'}, $self->{'mode'} );
    }

    defined( $self->{'fh'} ) || die("Failed to open {$self->{'filename'}}: $!");
    return $self;
}

sub close {
    my ($self) = @_;
    if ( defined( $self->{'fh'} ) ) {
        $self->{'fh'}->close || die("Fail to close file: $self->{'filename'}");
        $self->{'fh'} = undef;
    }
    return 1;
}

sub seek {
    my ( $self, $pos, $whence ) = @_;
    die("No filehandle open for {$self->{'filename'}")
      if !defined( $self->{'fh'} );
    if ( !defined($whence) ) {
        $whence = IO::File::SEEK_SET;
    }

    # TODO add protect for seek on gzip by performing close-open-seek?
    $self->{'fh'}->seek( $pos, $whence );
    return $self;
}

sub tell {
    my ( $self, @args ) = @_;
    die("No filehandle open for {$self->{'filename'}")
      if !defined( $self->{'fh'} );
    return $self->{'fh'}->tell(@args);
}

sub __DESTROY {
    my ($self) = @_;
    return $self->close();
}

sub getline {
    my ($self) = @_;
    return $self->{'fh'}->getline();
}

sub print {
    my ( $self, @args ) = @_;
    return $self->{'fh'}->print(@args);
}

#sub readlines {
#    my $lines;
#    if (1) {
#        return $lines;
#    }
#    return;
#}

1;
