package Logaholic::Log::Domlog;

use strict;
use warnings;

use Cwd qw(abs_path);
use Digest::MD5 qw(md5_hex);
use Time::Local qw(timegm);
use URI;
use URI::Escape ();
use URI::QueryParam;

my $MONTHS = {
    "Jan" => 0,
    "Feb" => 1,
    "Mar" => 2,
    "Apr" => 3,
    "May" => 4,
    "Jun" => 5,
    "Jul" => 6,
    "Aug" => 7,
    "Sep" => 8,
    "Oct" => 9,
    "Nov" => 10,
    "Dec" => 11,
};

sub parse {
    my ( $regex, $raw_line ) = @_;
    my $line = _parse_log_line_to_hash( $regex, $raw_line );
    return if !$line->{'host'};

    $line->{'bytes'} = 0 if ( !$line->{'bytes'} || $line->{'bytes'} eq '-' );

    parse_and_update_datetime($line);

    $line->{'crawl'} = 0;
    if ( $line->{'agent'} =~ /(bot|spider|crawl|slurp)$/i ) {
        $line->{'crawl'} = 1;
    }
    $line->{'agent_md5'} = md5_hex( $line->{'agent'} || '' );

    return $line;
}

sub _parse_log_line_to_hash {
    my ( $regex, $raw_line ) = @_;

    if ( ref $regex ne 'Regexp' ) {
        die('Invalid regex provide. Cannot parse log lines');
    }

    my @matches;
    @matches = ($raw_line =~ $regex);

    #tidyoff
    my @possible_fields = qw(
      host
      identuser
      authuser
      date time timezone
      method url protocol
      status
      bytes
      referrer
      agent
      cookie
    );

    #tidyon
    my $i = 0;
    my %line;
    foreach my $key (@possible_fields) {
        $line{$key} = (defined($matches[$i]) ) ? $matches[$i] : '';
        ++$i;
    }
    
#    if (!$line{'identuser'})
    return \%line;
}

sub parse_and_update_datetime {
    my ($line) = @_;
    my @date_parts = split( '/', $line->{'date'} );
    my @time_parts = split( ':', $line->{'time'} );

    $line->{'datetime'} = {
        'day' => $date_parts[0] || 00,
        'mon' => $MONTHS->{ $date_parts[1] || 'Jan' },
        'yr'  => $date_parts[2] || 0000,
        'hr'  => $time_parts[0] || 00,
        'min' => $time_parts[1] || 00,
        'sec' => $time_parts[2] || 00,
    };

    $line->{'gmt'} = timegm(
        $line->{'datetime'}->{'sec'}, $line->{'datetime'}->{'min'},
        $line->{'datetime'}->{'hr'},  $line->{'datetime'}->{'day'},
        $line->{'datetime'}->{'mon'}, $line->{'datetime'}->{'yr'}
    ) || 0;
    
    
    #NOTE: we're only working with timezone consideration AFTER we
    # aquire a timestamp...any further work do with 'datetime' will
    # be inaccurate.
    
    $line->{'timezone'} ||= 0000; 
    $line->{'timezone'} =~ m/^([\+-])?(\d\d)(\d\d)/;
    if ($1 eq '+') {
    	# subtract to make gmt
    	$line->{'gmt'} -= ($2 * 3600);
    	if ($3) {
    		$line->{'gmt'} -= ($3 * 60);
    	}
    }
    elsif ($1 eq '-') {
    	# add to make gmt
        $line->{'gmt'} += ($2 * 3600);
        if ($3) {
            $line->{'gmt'} += ($3 * 60);
        }
    }

    return $line;
}

sub get_normalized_string {
    my ($line)      = @_;
    my @ordering    = _normalized_string_list();
    my $line_string = join( '|LWA|',
        map { ( defined( $line->{$_} ) ) ? ( $line->{$_} ) : ('') } @ordering );
    return $line_string || '';
}

sub _normalized_string_list {
    return qw(
      host
      identuser
      authuser
      gmt
      timezone
      url
      status
      bytes
      referrer
      agent
      agent_md5
      cookie
      urlparams
      refparams
      keywords
      visitorid
      country
      crawl
      sessioncounter
    );
}

# pass a list of domains separated by a comma.  each item in list can be a
# regular string or a string to be interpreted as a regex
#   (these come from equivalent domain section of profile editor)
#
# by default, any schema will be dropped and a begin anchor added to the fqdn.tld
#  if the list item does not appear to be a fqdn [aka a uri path], say like
#  '/index.php' [which really doesn't make sense], then an anchor is not added
#  and we assume that your doing something explicit, albeit queer
sub domains_to_qr {
    my ( $csv, $quasi_anchor_uri ) = @_;
    $csv ||= '';
    $csv =~ s/\s*//g;
    $quasi_anchor_uri = ( defined($quasi_anchor_uri) ) ? $quasi_anchor_uri : 1;
    my @new_list;
    foreach my $item ( split( ',', $csv ) ) {
        next if !$item;
        if ( $quasi_anchor_uri && $item =~ m|([^:/]+://)?([^://].*)| ) {
            $item = qr/^($2)/;
        }
        else {
            $item = qr/(${item})/;
        }
        push @new_list, $item;
    }
    my $qr_str = join( '|', @new_list );
    return qr/$qr_str/;
}

sub URLDecode {
    my ($url) = @_;
    $url ||= '';
    $url =~ s/^\s+//;
    $url =~ s/\s+$//;
    $url =~ s/<!--(.|\n)*-->//g;
    $url = URI::Escape::uri_unescape($url);
    $url =~ tr/+/ /;
    
    return $url;
}

# currently not in use
sub URLEncode {
    my ($url) = @_;
    $url =~ s/([\W])/"%" . uc(sprintf("%2.2x",ord($1)))/eg;
    return $url;
}

# This subroutine trims whitespace from a string.
sub trim {
    my ($string) = @_;
    $string =~ s/^\s+//;
    $string =~ s/\s+$//;
    return $string;
}

sub get_logformat_regex {
    my ($logformat) = @_;

    # all groups share these
    #  ApacheCommonNoReferrerLogParser uses only these
    #tidyoff
    # ?<host>
    # ?<identuser>
    # ?<authuser>
    # ?<date> ?<time> ?<timezone>
    # ?<method> ?<url> ?<protocol>
    # ?<status>
    # ?<bytes>
    my @groups = qw(
      (\S+)
      (\S+)
      (\S+)
      \[([^:]+):(\d+:\d+:\d+)\s([^\]]+)\]
      "([^"]\S+)\s+([^"]+?)\s+([^"]\S+)"
      (\S+)
      (\S+)
    );

    #tidyon

    if (   $logformat eq 'ApacheCombinedLogParser'
        || $logformat eq 'LooseApacheCommonLogParser' )
    {
        # + referrer + agent - cookie
        # ?<referrer> ?<agent>
        push @groups, qw( "([^"]+)" "([^"]+)" );
    }
    elsif ( $logformat eq 'ApacheCombinedCookieLogParser' ) {

        # + referrer + agent + cookie
        # ?<referrer> ?<agent> ?<cookie>
        push @groups,
          qw( "([^"]+)" "([^"]+)" "([^"]+)" );
    }
    my $pattern = join( '\s+', @groups );
    return qr/\A$pattern\z/;
}

1;
