#!/usr/bin/perl

use strict;
use warnings;

use lib ('../');

use Test::More tests => 31;

use Logaholic::Preparse;
use Logaholic::Log::Domlog;

note('_munge_uri_against_filterhash() with domain authority regex');
{
    my $filter = { '/index.php' => [ 'key1', 'key3' ], };

    my $csv = 'dave.com,(www\.)?david\.com,lild.com';

    my $domain_qr = Logaholic::Log::Domlog::domains_to_qr($csv);
    {
        my $preparser = Logaholic::Preparse->new();
        my ( $path, $query, $authority );
        my $real_refer =
          'http://google.com/index.php?key1=value1&key2=value2&key3=value3';

        ( $path, $query, $authority ) =
          $preparser->_munge_uri_against_filterhash( $real_refer, $filter,
            $domain_qr );
        note(' -URI does not match authority domain regex');
        is( $path, '/index.php', 'path properly returned as bare path' );
        is( $authority, 'google.com',
            'authority returned as expected for schema://fqdn.tld' );
        like( $query, qr/(key2=value2)/,
"non user domain [schema://fqdn] does not have filter applied to query params"
        );
    }

    {
        my $preparser = Logaholic::Preparse->new();
        $preparser->{'strip_url_param_filter'} =
          { 'params' => ['key2'], 'mode' => 'exclude' };

        my ( $path, $query, $authority );
        my $logaholic_refer =
          'http://david.com/index.php?key1=value1&key2=value2&key3=value3';

        ( $path, $query, $authority ) =
          $preparser->_munge_uri_against_filterhash( $logaholic_refer, $filter,
            $domain_qr );
        note(' -URI matches authority domain regex');
        is(
            $path,
            '/index.php?key1=value1&key3=value3',
            'path properly returned with important params appended'
        );
        is( $authority, 'david.com',
            'authority returned as expected for schema://fqdn.tld' );
        unlike( $query, qr/(key2=value2)/,
            "user domain [schema://fqdn] has applied filter to query params" );
    }

    {
        my $preparser = Logaholic::Preparse->new();
        $preparser->{'strip_url_param_filter'} =
          { 'params' => ['key2'], 'mode' => 'exclude' };
        my ( $path, $query, $authority );
        my $eqv_refer =
          'http://dave.com/index.php?key1=value1&key2=value2&key3=value3';

        ( $path, $query, $authority ) =
          $preparser->_munge_uri_against_filterhash( $eqv_refer, $filter,
            $domain_qr );
        note(' -URI matches authority domain regex');
        is(
            $path,
            '/index.php?key1=value1&key3=value3',
            'path properly returned'
        );
        is( $authority, 'dave.com',
            'authority returned as expected for schema://fqdn.tld' );
        unlike( $query, qr/(key2=value2)/,
"another user domain [schema://fqdn] has applied filter to query params"
        );
    }
    {
        my $preparser = Logaholic::Preparse->new();
        my ( $path, $query, $authority );
        my $eqv_refer =
          'http://dave.com/index.php?key1=value1&key2=value2&key3=value3';

        ( $path, $query, $authority ) =
          $preparser->_munge_uri_against_filterhash( $eqv_refer, $filter,
            $domain_qr );
        note(' -URI matches authority domain regex, no strip filter');
        is(
            $path,
            '/index.php?key1=value1&key3=value3',
            'path properly returned'
        );
        is( $authority, 'dave.com',
            'authority returned as expected for schema://fqdn.tld' );
        like( $query, qr/(key2=value2)/,
"another user domain [schema://fqdn] has applied filter to query params"
        );
    }
    {
        my $preparser = Logaholic::Preparse->new();
        $preparser->{'strip_url_param_filter'} =
          { 'params' => ['key2'], 'mode' => 'exclude' };

      # scenario possible within original code that his function was ported from
      # - this might still be a possibility due to prior uri munging, but
      #   like has been removed from perl
        my ( $path, $query, $authority );
        my $quasi_refer =
          '://lild.com/index.php?key1=value1&key2=value2&key3=value3';

        ( $path, $query, $authority ) =
          $preparser->_munge_uri_against_filterhash( $quasi_refer, $filter,
            $domain_qr );
        note(
' -URI matches authority domain regex, despite URI protocol being broken'
        );
        is(
            $path,
            '/index.php?key1=value1&key3=value3',
            'path properly returned'
        );
        is( $authority, 'lild.com', 'authority always is simpley fqdn.tld' );
        unlike( $query, qr/(key2=value2)/,
            "user domain [://fqdn] has applied filter" );
    }
    {
        my $preparser = Logaholic::Preparse->new();

        # this illustrates that you wish to work against the domlog request uri,
        #  you should not pass the domain_authority regex.  only pass a domain
        #  authority regex if you what to filter bases on the uri path's domain
        # (which typically is absent in Apache log query uri)
        my ( $path, $query, $authority );
        my $not_a_refer = '/index.php?key1=value1&key2=value2&key3=value3';

        ( $path, $query, $authority ) =
          $preparser->_munge_uri_against_filterhash( $not_a_refer, $filter,
            $domain_qr );

        is( $path,      '/index.php', 'path properly returned' );
        is( $authority, '',           'authority is undef for path-only uri' );
        like( $query, qr/(key2=value2)/,
            "path-only does not have filter applied to query params" );
    }

}

note('_munge_uri_against_filterhash() WITHOUT domain authority regex');
{
    my $filter = { '/index.php' => [ 'key1', 'key3' ], };

    {
        my $preparser = Logaholic::Preparse->new();
        $preparser->{'strip_url_param_filter'} =
          { 'params' => ['key2'], 'mode' => 'exclude' };
        my ( $path, $query, $authority );
        my $real_refer =
          'http://google.com/index.php?key1=value1&key2=value2&key3=value3';

        ( $path, $query, $authority ) =
          $preparser->_munge_uri_against_filterhash( $real_refer, $filter );
        note(' -foreign domain parsed without regard to domain cause no regex');
        is(
            $path,
            '/index.php?key1=value1&key3=value3',
            'path properly returned'
        );
        is( $authority, 'google.com',
            'authority returned as expected for schema://fqdn.tld' );
        unlike( $query, qr/(key2=value2)/,
"non user domain [schema://fqdn] does have filter applied to query params"
        );
    }

    {
        my $preparser = Logaholic::Preparse->new();
        $preparser->{'strip_url_param_filter'} =
          { 'params' => ['key2'], 'mode' => 'exclude' };
        my ( $path, $query, $authority );
        my $logaholic_refer =
          'http://david.com/index.php?key1=value1&key2=value2&key3=value3';

        ( $path, $query, $authority ) =
          $preparser->_munge_uri_against_filterhash( $logaholic_refer,
            $filter );
        note(' -domain parsed without regard to domain cause no regex');
        is(
            $path,
            '/index.php?key1=value1&key3=value3',
            'path properly returned'
        );
        is( $authority, 'david.com',
            'authority returned as expected for schema://fqdn.tld' );
        unlike( $query, qr/(key2=value2)/,
            "user domain [schema://fqdn] has applied filter to query params" );
    }

    {
        my $preparser = Logaholic::Preparse->new();
        $preparser->{'strip_url_param_filter'} =
          { 'params' => ['key2'], 'mode' => 'exclude' };

      # scenario possible within original code that his function was ported from
      # - this might still be a possibility due to prior uri munging, but
      #   like has been removed from perl
        my ( $path, $query, $authority );
        my $quasi_refer =
          '://lild.com/index.php?key1=value1&key2=value2&key3=value3';

        ( $path, $query, $authority ) =
          $preparser->_munge_uri_against_filterhash( $quasi_refer, $filter );
        note(
' -domain parsed without regard to domain cause no regex, despite URI protocol being broken'
        );
        is(
            $path,
            '/index.php?key1=value1&key3=value3',
            'path properly returned'
        );
        is( $authority, 'lild.com', 'authority always is simpley fqdn.tld' );
        unlike( $query, qr/(key2=value2)/,
            "user domain [://fqdn] has applied filter to query params" );
    }
}

note('apply_strip_filter_to_params_hash');
{
    my $params = {
        'key1' => 'v1',
        'key2' => 'v2',
        'key3' => 'v3',
        'key4' => 'v4',
    };

    {
        my $strip_filter = {
            'params' => [ 'key1', 'key2' ],
            'mode'   => 'include',
        };

        my $actual =
          Logaholic::Preparse::apply_strip_filter_to_params_hash( $strip_filter,
            $params );

        my @actual_keys = sort( keys %{$actual} );
        my @actual_values = sort( values %{$actual} ) if values %{$actual};

        my @expected_keys   = sort(qw( key1 key2 ));
        my @expected_values = sort(qw( v1 v2 ));
        is_deeply( \@actual_keys, \@expected_keys,
            'strip filter properly applies keys for "include" mode' );
        is_deeply( \@actual_values, \@expected_values,
            'strip filter properly applies values to keys for "include" mode' );
    }

    {
        my $strip_filter = {
            'params' => [ 'key1', 'key2' ],
            'mode'   => 'exclude',
        };

        my $actual =
          Logaholic::Preparse::apply_strip_filter_to_params_hash( $strip_filter,
            $params );

        my @actual_keys = sort( keys %{$actual} );
        my @actual_values = map { ( $actual->{$_} ) } @actual_keys;

        my @expected_keys   = sort(qw( key3 key4 ));
        my @expected_values = sort(qw( v3 v4 ));
        is_deeply( \@actual_keys, \@expected_keys,
            'strip filter properly applies keys for "exclude" mode' );
        is_deeply( \@actual_values, \@expected_values,
            'strip filter properly applies values to keys for "exclude" mode' );
    }
}
