#!/usr/bin/perl

use strict;
use warnings;

use lib ('../');
use File::Temp;
use FindBin;
use Cwd qw(abs_path);
use JSON::Syck;

use Data::Dumper;

BEGIN {
    my $component_import_dir = abs_path( $FindBin::Bin . '/../../' );
    require "$component_import_dir/preparse.pl";
}

use Test::More;
use Data::Diff;

use Logaholic::Preparse;
use Logaholic::Log::Domlog;

my $test_data_dir = abs_path( $FindBin::Bin . '/data' );

my $cli_args = {
    'profile_config' => '',
    'schema_version' => 'v1',
    'output'         => 'JSON',
};

# Sub for creating temporary profile config that points to temporary input and
#  output files. Temp input is filled with lines from a sample log
sub make_domlog_setup {
    my ( $config, $inputfile, $outputfile ) =
      ( File::Temp->new(), File::Temp->new(), File::Temp->new(), );
    my ( $i_file, $o_file ) = (

        #$test_data_dir .'/test.domain.domlog',
        $inputfile->filename(),
        $outputfile->filename()
    );
    close($outputfile);

    my $data = <<CONFIG;
inputfile=$i_file
outputfile=$o_file
logformat=ApacheCombinedCookieLogParser
confdomain=www.logaholic.com
equivdomains=://logaholic.com
vidm=VIDM_IPPLUSAGENT
lastpos=0
firstlogline=0
skiptime=
googleparams=q|start|gclid|as_q|as_epq|as_oq|as_eq|as_sitesearch|as_rq|as_lq
skipfiles=.gif|.jpg|.jpeg|.js|.css|.txt|.ico|.swf|.xml|.png|.dll|logaholic/
skipips=
importantparams=
urlparamfiltermode=Exclude
urlparamfilter=
CONFIG
    print {$config} $data;
    close($config);

    open( my $domlog_fh, '<', $test_data_dir . '/www.test.log' )
      || die $!;
    while ( my $line = readline($domlog_fh) ) {
        chomp $line if $line;
        print {$inputfile} "$line\n" if $line;
    }
    close($domlog_fh);
    close($inputfile);

    return ( $config, $inputfile, $outputfile );
}

note('Parse www.test.log');
{
    my ( $config, $inputfile, $outputfile ) = make_domlog_setup();
    $cli_args->{'profile_config'} = $config;

    my $actual = Logaholic::script::preparse->script($cli_args);
    my $expect = {
        'bandwidth' => {
            '2012-11-30' => 4266153,
        },
        'lastlogpos' => 351109,
        'firstlogline' =>
'213.206.113.136 - - [30/Nov/2012:00:00:01 +0100] "GET /cerberus/cron.php HTTP/1.0" 200 - "-" "Wget/1.10.2 (Red Hat modified)" "-"' . "\n",
    };

    is_deeply( $actual, $expect,
        "Can parse domlog based on profile and get proper stats" );

    my $parsed_stub = $test_data_dir . '/oldperl_lwa_log';
    
    compare_parsed_logs($parsed_stub, $outputfile);
}

####### FUNCTIONS #########

sub domlog_elements {
    return qw(
      host
      identuser
      authuser
      gmt
      timezone
      url
      status
      bytes
      referrer
      agent
      agent_md5
      cookie
      urlparams
      refparams
      keywords
      visitorid
      country
      crawl
      sessioncounter
    );
}

sub compare_parsed_logs {
    my ($stub, $output) = @_;
    my $diff_output = `diff -y -W 999999 $stub $output`;
    my $fh;
    open($fh, '>', '/tmp/test.domain.domlog.diff') || die $!;
    print ${fh} $diff_output;
    close($fh);
    open(my $fhr, '<', '/tmp/test.domain.domlog.diff') || die $!;
    my $i = 0;
    my $investigate = [];
    while (readline($fhr)) {
        my $line = $_;
        my @lines = split(/\s+\|\s+/, $line, 2);
        next if !$lines[1];
        chomp $lines[1];
        push @{$investigate}, [$i, $lines[0], $lines[1]];
        ++$i;
    }
    close ($fhr);
    my @fields = domlog_elements();
    my $fields_column_count = scalar @fields; 
    my $total_failed_lines = 0;
    my $total_points_of_failure = 0;
    foreach my $disc (@{$investigate}) {
        my @stub_parts = split(/\|LWA\|/, $disc->[1], $fields_column_count);
        my @actual_parts = split(/\|LWA\|/, $disc->[2], $fields_column_count);
        my @fails;
        for (my $i=0; $i < $fields_column_count; ++$i) {
            my $got = '';
            my $expect = '';
            my $text = '';
            if ($fields[$i] =~ m/params/) {
                
                my $a_params = (defined($actual_parts[$i])) ? $actual_parts[$i] : '';
                if ($a_params && $a_params !~ /not\sprovided/i && $a_params !~ /^\?/) {
                    push @fails, ["actual '$fields[$i]' string starts with '?'"];
                }
                $a_params =~ s/^\?+//;
                
                my $s_params = (defined($stub_parts[$i])) ? $stub_parts[$i] : '';
                if ($s_params && $s_params !~ /not\sprovided/i && $s_params !~ /^\?/ ) {
                    push @fails, ["stub '$fields[$i]' string starts with '?'"] ;
                }
                $s_params =~ s/^\?+//;
                
                my ($actual_params, $stub_params) = ({},{});
                my @keyvalues = split(/\&/, $a_params);
                foreach my $keyvalue (@keyvalues) {
                    next if $keyvalue eq qq//;
                    my ($key, $value) = split(/=/, $keyvalue, 2);
                    $actual_params->{$key} = $value;
                }
                
                @keyvalues = split(/&/, $s_params);
                foreach my $keyvalue (@keyvalues) {
                    next if ! $keyvalue;
                    my ($key, $value) = split(/=/, $keyvalue, 2);
                    $stub_params->{$key} = $value;
                }

                $got = $actual_params;
                $expect = $stub_params;
                $text = "field '$fields[$i]' has identical key/values";
            }
            else {
                $got = $actual_parts[$i];
                $expect = $stub_parts[$i];
                $text = "field '$fields[$i]' has identical values";
            }

            my $out = Data::Diff::Diff( $expect, $got);

            my $diff_str = '';

            if (defined($out->{'type'})) {
                delete $out->{'type'};
            }
            if (defined($out->{'same'})) {
                delete $out->{'same'};
            }
            
            my $keys = scalar keys $out;
            if (!$keys ){
                $diff_str = '';
            }
            else{
                $diff_str = Dumper $out;
            }

            if ($diff_str) {
                $diff_str =~ s/\$VAR1 = /     /;
                $diff_str =~ s/    / /g;
                $diff_str =~ s/};\Z/}/;
                push @fails, [$text, $diff_str];
            }
        }

        if (@fails) {
            my $str = "\n\n#######################################\n"
                    . "### Line discrepancy $disc->[0]  ###\n"
                    . "#######################################\n";
            diag($str);
            
            foreach ( @fails ) {
                fail($_->[0]);
                diag($_->[1]) if (exists $_->[1]);
            }
            
            diag("\n". 'EXPECTED (a): ' . $disc->[1]);
            diag('GOT (b):      ' . $disc->[2]);
            $total_failed_lines++;
            $total_points_of_failure += scalar @fails;
        }
        
    }

    my $total_str = "\n\n\n##########################################################\n"
                  . "##########################################################\n"
                  . "Test stub line count and filename (diff_a, uniq_a):\n"
                  . `wc -l $stub`
                  . "Test parsed line count and filename (diff_b, uniq_b):\n"
                  . `wc -l $output`
                  . "Total lines that needed investigation:   " . scalar @{$investigate} . "\n"
                  . "Total log lines with real discrepancies: " . $total_failed_lines . "\n"
                  . "Total discrepancy points:                " . $total_points_of_failure . "\n"
                  . "##########################################################";
    diag($total_str);
}
done_testing();