#!/usr/bin/perl

package Logaholic::script::preparse;

use strict;
use warnings;

use FindBin;
use Digest::MD5 qw(md5_hex);
use lib ("$FindBin::Bin/Perl");
use Getopt::Long;
use Pod::Usage;

use Logaholic::Preparse;
use Logaholic::Log::Domlog;

our $VERSION = '1.00';

unless ( caller() ) {
    my $man           = 0;
    my $help          = 0;
    my $format        = '';
    my $whats_version = 0;

    my $exit_value = 0;
    my $args       = {};

    GetOptions(
        $args,
        'profile_config=s',
        'schema_version=s',
        'output_format=s' => \$format,
        'help|?'          => \$help,
        'man'             => \$man,
        'version'         => \$whats_version,
    ) || pod2usage(2);

    pod2usage(1) if $help;
    pod2usage( -existatus => 0, -verbose => 2 ) if $man;
    if ($whats_version) {
        print $VERSION . "\n";
        exit(0);
    }
    if (   !defined( $args->{'profile_config'} )
        || !defined( $args->{'schema_version'} ) )
    {
        pod2usage(1);
    }

    my $result = __PACKAGE__->script($args);
    my $output = __PACKAGE__->get_output_string( $result, $format );
    if ( ref $result ne 'HASH' || defined $result->{'errors'} ) {
        print STDERR $output;
        exit(1);
    }

    print $output;
    exit(0);
}

sub script {
    my ( $self, $args ) = @_;

    my $preparser = Logaholic::Preparse->new( $args->{'schema_version'} );
    $preparser->load_config_from_filename( $args->{'profile_config'} );

    $preparser->enable_profile_timezone_warping();
    
    ## determine if we should start at the begin
    # if ( first line md5 matches md5 of last known line )
    #  then start at last known line
    # else
    #  then start at first line

    my $previous_first_log_line_md5 =
      $preparser->get_config_value('firstlogline');
    my $previous_last_log_position = $preparser->get_config_value('lastpos')
      || 0;

    my $original_fh = $preparser->getHandle('inputfile');
    $original_fh->open();
    $original_fh->seek(0);

    # NOTE: we're not chomping here
    my $current_first_log_line = $original_fh->readline();

    my $start_parsing_at = 0;
    if ( $previous_first_log_line_md5 eq md5_hex($current_first_log_line) ) {
        $start_parsing_at = $previous_last_log_position;
    }

    #TODO: ask about most appropriate default
    my $logformat_regex = Logaholic::Log::Domlog::get_logformat_regex(
        $preparser->get_config_value('logformat')
          || 'ApacheCombinedLogParser' );

    ## generate a normalized copy of original file
    # - parse original file line by line at previously determined starting
    #   point, retreiving a normalized structure
    # - print a line to our new file using normalized structure
    my $normalized_fh = $preparser->getHandle('outputfile');
    $normalized_fh->open();

    # We've read a line; seeking backwards in a gz is forbidden, so close &
    #  reopen if we aren't going to seek fowards
    if ( !$start_parsing_at ) {
        $original_fh->close();
        $original_fh->open();
    }
    else {
        $original_fh->seek($start_parsing_at);
    }

    while ( my $line = $original_fh->readline() ) {
        chomp $line if $line;
        my $line_data =
          Logaholic::Log::Domlog::parse( $logformat_regex, $line );
        if ( $preparser->is_valid_line($line_data) ) {
            $preparser->add_refinements_to_line_data($line_data);
            $preparser->update_statistics($line_data);
            $preparser->add_country_code($line_data);
            $normalized_fh->print(
                Logaholic::Log::Domlog::get_normalized_string($line_data)
                  . "\n" );
        }
    }
    $normalized_fh->close();

    my $current_last_log_position = $original_fh->tell();
    $original_fh->close;

    ## ensure proper privileges for new normalized file
    $preparser->secure_file('outputfile');

    return {
        'lastlogpos'   => $current_last_log_position,
        'firstlogline' => $current_first_log_line,
        'bandwidth'    => $preparser->{'total_bytes'},
        'sessioncounter'    => $preparser->{'sessioncounter'}
    };
}

sub get_output_string {
    my ( $self, $data, $format, $indent ) = @_;
    $data ||= {};
    $indent = ( defined($indent) ) ? $indent : '';
    my $output_string = '';
    $format ||= '';
    ##SOMEDAY replace with JSON::SYCK or something standard; will be required
   #  if the data is ever anything more than associative alphanumeric key/values
    if ( $format eq 'JSON' ) {
        $output_string .= "{\n";
        my @members;
        while ( my ( $key, $value ) = each( %{$data} ) ) {
            my $string = '';
            my $value_str;
            if ( ref $value eq 'HASH' ) {
                $value_str =
                  $self->get_output_string( $value, 'JSON', $indent . "\t" );
            }
            else {
                $value_str = '"' . $value . '"';
            }
            $string = $indent . '"' . $key . '":' . $value_str;
            push @members, $string;
        }
        $output_string .= join( ",\n", @members ) . "\n$indent}";
    }
    else {
        my @keys = sort keys %{$data};
        foreach my $key (@keys) {
            my $value = $data->{$key};
            if ( ref $value eq 'HASH' ) {
                my @sub_keys = sort keys %{$value};
                foreach my $k (@sub_keys) {
                    $output_string .= "$key:$k=" . $value->{$k} . "\n";
                }
            }
            else {
                $output_string .= "$key:$value\n";
            }
        }
    }
    $output_string =~ s/\n+/\n/mg;
    return $output_string;
}

1;
__END__

=head1 NAME

preparse.pl - Normalize a domlog for use by Loghaolic and it's MySQL routines 
based on a schema-enforced profile configuration file. 

=head1 SYNOPSIS

preparse.pl [options] [arguments]

 Arguments:
   -profile_config   Absolute path to profile configuration file
   -schema_version   Schema version used by profile configuration file
   -output_format    Format of output printed to STDOUT of this script
   
 Options:
   -help             Brief help message
   -man              Full documentation
   -version          Print the version of this utility

=head1 OPTIONS

=over 4

=item B<-help>

Print a brief help message and exits.

=item B<-man>

Prints the manual page and exits.

=item B<-version>

Print the version number of this utility script.

=back

=head1 ARGUMENTS

If information options are not used, then the script will need arguments to
perform its routine.

=over 4

=item B<-profile_config>

Absolute path to the profile configuration file that will be used by this script. 
[REQUIRED]

=item B<-schema_version>

The schema version that should be used when sourcing key/value information from
the named profile configuration file. [REQUIRED]

=item B<-output_format>

Optional argument that will alter the structure of the output generated by this
script.  By default this script uses a human/machine readable C<key:valueEOL>
format.  Valid options include: C<JSON>. [OPTIONAL]

=back

=head1 DESCRIPTION

B<preparse.pl> will read the named profile configuration file, gathering
relevant information based on the named schema it should have been written for.

Within the configuration file, there will be a specified domlog that this
script should normalized.  Also, the configuration file will name the expected
filename where the normalized data should be written.

The normalized data file is then processed by Logaholic (eg. subsequent routines 
that store domlog traffic, and analytical information thereof, within MySQL).

The script will output a summary that describes the first line processed, the
last line processed and total bandwidth bytes per day.  The format of this is
rudementary and can be parsed by a human or machine.  An optional format, 
I<JSON>, can be achieved by passing the argument C<-output_format JSON>.

=head1 HISTORY

The original, monolithic script was authored Logaholic staff as preparse.plx and
shipped as a part of Logaholic 4.0 BETA.

=head1 AUTHOR

David Neimeyer E<lt>F<davidneimeyer@gmail.com>E<gt>.

=head1 COPYRIGHT AND LICENSE

Copyright(c) 2012 Logaholic B.V.. All Rights Reserved.

L<http://www.logaholic.com/License.txt>
   
=cut
